#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
Some custom utility functions that we use
"""


class HostsCLIException(Exception):
    """
    A Custom Exception class
    """


class WebsiteImportError(HostsCLIException):
    """
    An error to raise when no Domain list found for given website
    """


class SudoRequiredError(HostsCLIException):
    """
    An error raised when the given hosts files is not writable.
    "sudo" permissions are required to run this command.
    """



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
List Supported Websites:
========================
List all websites
::
    $ hostscli websites


Block Websites
==============

To block facebook:
::
    $ sudo hostscli block facebook

To block facebook & youtube:
::
    $ sudo hostscli block facebook,youtube


Unblock Websites
================

To unblock facebook:
::
    $ sudo hostscli unblock facebook

To unblock facebook & youtube:
::
    $ sudo hostscli unblock facebook,youtube


Block All supported Websites
============================

Block all available
::
    $ sudo hostscli block_all

Use `--ignore` ot `-i` to ignore websites while blocking.
::
    $ sudo hostscli block_all -i facebook,google


Unblock All supported Websites
==============================

Ubnlock all supported
::
    $ sudo hostscli unblock_all

Use `--ignore` ot `-i` to ignore websites while ubblocking.
::
    $ sudo hostscli unblock_all -i facebook,google

"""

from click import echo, group, argument, option

from hostscli.utils import get_websites, block as _block, unblock as _unblock


@group()
def cli():
    """
    hostscli - A Simple tool to block/unblock websites using hosts file.
    """


@cli.command()
def websites():
    """
    List all available websites
    """
    echo("Available Websites: \n")
    websites = get_websites()
    for website in websites:
        echo(website)


@cli.command()
@argument('websites')
def block(websites):
    """
    Block specific website(s)
    """
    for website in websites.split(","):
        _block(website)
        echo('Blocked %s!' % website)


@cli.command()
@argument('websites')
def unblock(websites):
    """
    Unblock specific website(s)
    """
    for website in websites.split(","):
        _unblock(website)
        echo('%s unblocked!' % website)


@cli.command()
@option('--ignore', '-i', default="", help='Ignore websites while blocking')
def block_all(ignore):
    """
    Block all available websites
    """
    websites = get_websites()
    ignore_websites = ignore.split(",")
    for website in websites:
        if website not in ignore_websites:
            _block(website)


@cli.command()
@option('--ignore', '-i', default="", help='Ignore websites while unblocking')
def unblock_all(ignore):
    """
    Unblock all available websites
    """
    websites = get_websites()
    ignore_websites = ignore.split(",")
    for website in websites:
        if website not in ignore_websites:
            _unblock(website)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""



Installation
============

    pip install -U hostscli


Help
====

    Type `hostscli --help` to get a help on all the available commands


Usage
=====

**Please note that we require `sudo` permissions to change `/etc/hosts` file**

.. automodule:: hostscli.cli
    :members:
    :undoc-members:
    :show-inheritance:


Built-in utility functions
==========================

.. automodule:: hostscli.utils
    :members:
    :undoc-members:
    :show-inheritance:


Custom Errors
=============

.. automodule:: hostscli.errors
    :members:
    :undoc-members:
    :show-inheritance:


Indices and tables
==================

* :ref:`genindex`
* :ref:`modindex`
* :ref:`search`
"""



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
Some utility functions that gets our work done
"""

from functools import wraps
from importlib import import_module
from os import listdir, access, W_OK

from hostscli.errors import WebsiteImportError, SudoRequiredError
from hostscli.constants import HOSTS_FILE, FORMAT, WEBSITES_PACKAGE, \
    IMPORT_ERROR, ROOT_ERROR, IGNORE_WEBSITES


def hosts_write_access(f):
    """
    A Decorator to check if the given hosts file is writeable or not.
    If not writeable, it raises a `SudoRequiredError`.
    """
    @wraps(f)
    def wrapper(website, hosts_file=HOSTS_FILE):
        if not access(hosts_file, W_OK):
            raise SudoRequiredError(ROOT_ERROR)
        return f(website, hosts_file)
    return wrapper


def get_websites():
    """
    Get a list of available websites
    """
    websites_path = import_module(WEBSITES_PACKAGE)
    websites_path = websites_path.__file__.split("__init__")[0]
    websites = listdir(websites_path)
    for ignore_website in IGNORE_WEBSITES:
        if ignore_website in websites:
            websites.remove(ignore_website)
    return list(set([website.split(".")[0] for website in websites]))


def get_lines(website):
    """
    Get a list of lines of a specific website
    to append to / remove from the hosts files

    raise `WebsiteImportError` if a website is not available
    """
    website = website.lower()
    try:
        module = import_module('%s.%s' % (WEBSITES_PACKAGE, website))
        return [FORMAT % domain for domain in module.DOMAINS]
    except ImportError:
        raise WebsiteImportError(IMPORT_ERROR % website)


@hosts_write_access
def block(website, hosts_file):
    """
    Add entries into the host file to block specific websites
    """
    target_lines = get_lines(website)
    with open(hosts_file, 'a') as hosts_file:
        for target_line in target_lines:
            hosts_file.write(target_line)
    return target_line


@hosts_write_access
def unblock(website, hosts_file):
    """
    Remove entries from the host file to unblock specific websites
    """
    target_lines = get_lines(website)
    input_lines = open(hosts_file, "r").readlines()
    with open(hosts_file, "w") as hosts_file:
        for input_line in input_lines:
            if input_line not in target_lines:
                hosts_file.write(input_line)
    return target_lines



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
File name: constants.py
Author: dhilipsiva <dhilipsiva@gmail.com>
Date created: 2016-12-29
"""

from os import environ

FORMAT = '127.0.0.1 %s\n'
WEBSITES_PACKAGE = 'hostscli.websites'
HOSTS_FILE = environ.get('HOSTS_FILE', '/etc/hosts')

IMPORT_ERROR = """

No Domain list found for website: %s

Please raise a Issue here: https://github.com/dhilipsiva/hostscli/issues/new
if you think we should add domains for this website.

type `hostscli websites` to see a list of websites that you can block/unblock


"""

ROOT_ERROR = """

"sudo" permissions are required to run this command.

Please run the last command again with sudo

"""
IGNORE_WEBSITES = ["__pycache__", "__init__.py", "__init__.pyc"]


# TEST CONSTANTS

TEST_WEBSITE = 'test'
TEST_LINE = '127.0.0.1 test.com\n'
